package org.jcg.zheng.demo.async;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ForkJoinPool;

import org.jcg.zheng.demo.async.data.DataUtils;
import org.jcg.zheng.demo.async.data.Order;
import org.jcg.zheng.demo.async.tasks.SupplierOrderTask;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ProcessOrders_CompletableFuture {

    ForkJoinPool fjp = new ForkJoinPool(10);

    final Logger logger = LoggerFactory.getLogger(this.getClass());

    public void processOrders(int orderCounts) {
        logger.info(DataUtils.THREAD_STARTS);
        List<CompletableFuture<Order>> futures = new ArrayList<>();

        for (int i = 0; i < orderCounts; i++) {
            SupplierOrderTask order = new SupplierOrderTask(i);

            CompletableFuture<Order> future = CompletableFuture.supplyAsync(order, fjp)
                    .thenApply(Order::emailNotification).whenComplete((o, r) -> {
                        logger.info("Send email for " + o.toString());
                    });

            futures.add(future);
            logger.info("future isDone=" + future.isDone());
        }
    }

    public void processOrders_chain(int orderCounts) {
        logger.info(DataUtils.THREAD_STARTS);


        List<CompletableFuture<Order>> futures = new ArrayList<>();

        for (int i = 0; i < orderCounts; i++) {
            CompletableFuture<Order> future = new CompletableFuture<>();
            chainOrderProcess(future);
            Order order = new Order();
            order.setOrderId(i);
            boolean completed = future.complete(order);
            logger.info("future completed=" + completed);
            futures.add(future);
        }

    }

    // process order, then notify user via an email, then output
    private void chainOrderProcess(CompletableFuture<Order> future) {
        logger.info(DataUtils.THREAD_STARTS);
        future.thenApply(Order::process).thenApply(Order::emailNotification)
                .thenAccept(System.out::println);
        logger.info(DataUtils.THREAD_COMPLETED);
    }

}
