package org.jcg.zheng.demo.async;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;

import org.jcg.zheng.demo.async.tasks.RunnableTask;
import org.jcg.zheng.demo.async.tasks.SupplierTask;
import org.junit.Test;

public class CompletableFutureHandleErrorTest extends TestBase {
    private Runnable runableTask = new RunnableTask();
    private SupplierTask supplierTask = new SupplierTask();

    @Test
    public void swallow_exception() throws InterruptedException, ExecutionException {
        CompletableFuture<Integer> future = CompletableFuture.supplyAsync(supplierTask);

        future.exceptionally(e -> swallowError(e)).thenApply(old -> old * 2)
                .thenApply(old -> old + 2).thenAccept(d -> printNum(d));

        future.completeExceptionally(new RuntimeException("Oops!")); // TODO not fully understand
        // should print out 202 = 100 * 2 + 2
    }

    @Test
    public void test_exception_2() {
        CompletableFuture<Integer> future = CompletableFuture.supplyAsync(supplierTask);

        future.exceptionally(e -> throwError(e)).thenApply(old -> old * 2).thenApply(old -> old + 2)
                .thenAccept(d -> printNum(d));

        future.completeExceptionally(new RuntimeException("Oops!"));
    }

    private int swallowError(Throwable e) {
        logger.info("ERROR:" + e.getMessage());
        return 100;
    }

    private int throwError(Throwable e) {
        logger.info("ERROR:" + e.getMessage());
        throw new RuntimeException("Cannot handle the error");
    }

    @Test
    public void test_cancel_InterruptableRunableTask_only_attempt_not_guaranteed() {
        // the async task still completes after cancel is sent
        CompletableFuture future = CompletableFuture.runAsync(runableTask);
        assertFalse(future.isDone());
        try {
            TimeUnit.SECONDS.sleep(1);
            boolean cancelStatus = future.cancel(true);

            assertTrue(future.isCancelled());
            assertTrue(cancelStatus);
        } catch (InterruptedException e) {
        }
    }

}
